
LIBRARY ieee ;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

entity EXPENDEDOR is
   port(
      clk             : in std_logic; -- clk
      reset           : in std_logic; -- reset
      nm              : in std_logic; -- pulso que indica nueva moneda
      R               : in std_logic_vector (7 downto 0); -- precio refresco
      M               : in std_logic_vector (7 downto 0); -- valor de moneda insertada
      exp             : out std_logic   -- pulso para expender refrecto
      );
end;

architecture RTL of EXPENDEDOR is
-- states
      type state_type IS
        (st_init, st_contar, st_expender);
      attribute enum_encoding : string;
      attribute enum_encoding of state_type : type is "one-hot"; -- "default", "sequential", "gray", "johnson", or "one-hot"
      signal STATE : state_type;
      -- info sobre codificación: http://quartushelp.altera.com/13.0/mergedProjects/hdl/vhdl/vhdl_file_dir_enum_encoding.htm

      signal A : std_logic_vector (7 downto 0); -- Acumulado de monedas insertadas
begin

  -- Process donde se generan
  -- las transferencias a memoria
  -- las transiciones de estado
   transf: process (reset, clk)
   begin
     if reset = '1' then
       STATE <= st_init;
     elsif clk'event and clk='1'then  
       case STATE is
         
        when st_init =>
          A <= (others => '0');
          STATE <= st_contar;

          
         when st_contar =>
            if nm = '1' then
               A <= A + M;
            end if;
            
            if A >= R then
               STATE <= st_expender;
            else
               STATE <= st_contar;
            end if;

        when st_expender =>
          A <= (others => '0');
          STATE <= st_contar;

        when others =>     
            --STATE <= st_init;

       end case;

       end if;
   end process;

  -- Process donde se generan las salidas
   salidas: process (STATE)
   begin
       case STATE is
        when st_init =>
          exp <= '0';
          
        when st_contar =>
          exp <= '0';

         -- when st_init | st_contar =>
         --   exp <= '0';
          
        when st_expender =>
          exp <= '1';

         when others =>
          exp <= '0';
          
       end case;

   end process;



end RTL;
